@@ -1,26 +1,6 @@
-/* Simple Plugin API
- *
- * Copyright © 2018 Wim Taymans
- *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice (including the next
- * paragraph) shall be included in all copies or substantial portions of the
- * Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
- */
+/* Simple Plugin API */
+/* SPDX-FileCopyrightText: Copyright © 2018 Wim Taymans */
+/* SPDX-License-Identifier: MIT */
 
 #ifndef SPA_VIDEO_RAW_H
 #define SPA_VIDEO_RAW_H
@@ -154,11 +134,12 @@ enum spa_video_format {
  * Extra video flags
  */
 enum spa_video_flags {
-	SPA_VIDEO_FLAG_NONE = 0,			/**< no flags */
-	SPA_VIDEO_FLAG_VARIABLE_FPS = (1 << 0),		/**< a variable fps is selected, fps_n and fps_d
-							 *   denote the maximum fps of the video */
-	SPA_VIDEO_FLAG_PREMULTIPLIED_ALPHA = (1 << 1),  /**< Each color has been scaled by the alpha value. */
-	SPA_VIDEO_FLAG_MODIFIER = (1 << 2),		/**< use the format modifier */
+	SPA_VIDEO_FLAG_NONE = 0,				/**< no flags */
+	SPA_VIDEO_FLAG_VARIABLE_FPS = (1 << 0),			/**< a variable fps is selected, fps_n and fps_d
+								 *   denote the maximum fps of the video */
+	SPA_VIDEO_FLAG_PREMULTIPLIED_ALPHA = (1 << 1),		/**< Each color has been scaled by the alpha value. */
+	SPA_VIDEO_FLAG_MODIFIER = (1 << 2),			/**< use the format modifier */
+	SPA_VIDEO_FLAG_MODIFIER_FIXATION_REQUIRED = (1 << 3),	/**< format modifier was not fixated yet */
 };
 
 /**
