@@ -24,6 +24,14 @@ extern "C" {
 #include <spa/pod/iter.h>
 #include <spa/pod/vararg.h>
 
+#ifndef SPA_API_POD_BUILDER
+ #ifdef SPA_API_IMPL
+  #define SPA_API_POD_BUILDER SPA_API_IMPL
+ #else
+  #define SPA_API_POD_BUILDER static inline
+ #endif
+#endif
+
 struct spa_pod_builder_state {
 	uint32_t offset;
 #define SPA_POD_BUILDER_FLAG_BODY	(1<<0)
@@ -51,20 +59,20 @@ struct spa_pod_builder {
 
 #define SPA_POD_BUILDER_INIT(buffer,size)  ((struct spa_pod_builder){ (buffer), (size), 0, {0,0,NULL},{NULL,NULL}})
 
-static inline void
+SPA_API_POD_BUILDER void
 spa_pod_builder_get_state(struct spa_pod_builder *builder, struct spa_pod_builder_state *state)
 {
 	*state = builder->state;
 }
 
-static inline void
+SPA_API_POD_BUILDER void
 spa_pod_builder_set_callbacks(struct spa_pod_builder *builder,
 		const struct spa_pod_builder_callbacks *callbacks, void *data)
 {
 	builder->callbacks = SPA_CALLBACKS_INIT(callbacks, data);
 }
 
-static inline void
+SPA_API_POD_BUILDER void
 spa_pod_builder_reset(struct spa_pod_builder *builder, struct spa_pod_builder_state *state)
 {
 	struct spa_pod_frame *f;
@@ -74,12 +82,12 @@ spa_pod_builder_reset(struct spa_pod_builder *builder, struct spa_pod_builder_st
 		f->pod.size -= size;
 }
 
-static inline void spa_pod_builder_init(struct spa_pod_builder *builder, void *data, uint32_t size)
+SPA_API_POD_BUILDER void spa_pod_builder_init(struct spa_pod_builder *builder, void *data, uint32_t size)
 {
 	*builder = SPA_POD_BUILDER_INIT(data, size);
 }
 
-static inline struct spa_pod *
+SPA_API_POD_BUILDER struct spa_pod *
 spa_pod_builder_deref(struct spa_pod_builder *builder, uint32_t offset)
 {
 	uint32_t size = builder->size;
@@ -91,7 +99,7 @@ spa_pod_builder_deref(struct spa_pod_builder *builder, uint32_t offset)
 	return NULL;
 }
 
-static inline struct spa_pod *
+SPA_API_POD_BUILDER struct spa_pod *
 spa_pod_builder_frame(struct spa_pod_builder *builder, struct spa_pod_frame *frame)
 {
 	if (frame->offset + SPA_POD_SIZE(&frame->pod) <= builder->size)
@@ -99,7 +107,7 @@ spa_pod_builder_frame(struct spa_pod_builder *builder, struct spa_pod_frame *fra
 	return NULL;
 }
 
-static inline void
+SPA_API_POD_BUILDER void
 spa_pod_builder_push(struct spa_pod_builder *builder,
 		     struct spa_pod_frame *frame,
 		     const struct spa_pod *pod,
@@ -115,21 +123,30 @@ spa_pod_builder_push(struct spa_pod_builder *builder,
 		builder->state.flags = SPA_POD_BUILDER_FLAG_FIRST | SPA_POD_BUILDER_FLAG_BODY;
 }
 
-static inline int spa_pod_builder_raw(struct spa_pod_builder *builder, const void *data, uint32_t size)
+SPA_API_POD_BUILDER int spa_pod_builder_raw(struct spa_pod_builder *builder, const void *data, uint32_t size)
 {
 	int res = 0;
 	struct spa_pod_frame *f;
 	uint32_t offset = builder->state.offset;
+	size_t data_offset = -1;
 
 	if (offset + size > builder->size) {
+		/* data could be inside the data we will realloc */
+		if (spa_ptrinside(builder->data, builder->size, data, size, NULL))
+			data_offset = SPA_PTRDIFF(data, builder->data);
+
 		res = -ENOSPC;
 		if (offset <= builder->size)
 			spa_callbacks_call_res(&builder->callbacks,
 					struct spa_pod_builder_callbacks, res,
 					overflow, 0, offset + size);
 	}
-	if (res == 0 && data)
+	if (res == 0 && data) {
+		if (data_offset != (size_t) -1)
+			data = SPA_PTROFF(builder->data, data_offset, const void);
+
 		memcpy(SPA_PTROFF(builder->data, offset, void), data, size);
+	}
 
 	builder->state.offset += size;
 
@@ -139,14 +156,14 @@ static inline int spa_pod_builder_raw(struct spa_pod_builder *builder, const voi
 	return res;
 }
 
-static inline int spa_pod_builder_pad(struct spa_pod_builder *builder, uint32_t size)
+SPA_API_POD_BUILDER int spa_pod_builder_pad(struct spa_pod_builder *builder, uint32_t size)
 {
 	uint64_t zeroes = 0;
 	size = SPA_ROUND_UP_N(size, 8) - size;
 	return size ? spa_pod_builder_raw(builder, &zeroes, size) : 0;
 }
 
-static inline int
+SPA_API_POD_BUILDER int
 spa_pod_builder_raw_padded(struct spa_pod_builder *builder, const void *data, uint32_t size)
 {
 	int r, res = spa_pod_builder_raw(builder, data, size);
@@ -155,7 +172,7 @@ spa_pod_builder_raw_padded(struct spa_pod_builder *builder, const void *data, ui
 	return res;
 }
 
-static inline void *spa_pod_builder_pop(struct spa_pod_builder *builder, struct spa_pod_frame *frame)
+SPA_API_POD_BUILDER void *spa_pod_builder_pop(struct spa_pod_builder *builder, struct spa_pod_frame *frame)
 {
 	struct spa_pod *pod;
 
@@ -172,7 +189,7 @@ static inline void *spa_pod_builder_pop(struct spa_pod_builder *builder, struct
 	return pod;
 }
 
-static inline int
+SPA_API_POD_BUILDER int
 spa_pod_builder_primitive(struct spa_pod_builder *builder, const struct spa_pod *p)
 {
 	const void *data;
@@ -198,13 +215,13 @@ spa_pod_builder_primitive(struct spa_pod_builder *builder, const struct spa_pod
 
 #define SPA_POD_INIT_None() SPA_POD_INIT(0, SPA_TYPE_None)
 
-static inline int spa_pod_builder_none(struct spa_pod_builder *builder)
+SPA_API_POD_BUILDER int spa_pod_builder_none(struct spa_pod_builder *builder)
 {
 	const struct spa_pod p = SPA_POD_INIT_None();
 	return spa_pod_builder_primitive(builder, &p);
 }
 
-static inline int spa_pod_builder_child(struct spa_pod_builder *builder, uint32_t size, uint32_t type)
+SPA_API_POD_BUILDER int spa_pod_builder_child(struct spa_pod_builder *builder, uint32_t size, uint32_t type)
 {
 	const struct spa_pod p = SPA_POD_INIT(size,type);
 	SPA_FLAG_CLEAR(builder->state.flags, SPA_POD_BUILDER_FLAG_FIRST);
@@ -213,7 +230,7 @@ static inline int spa_pod_builder_child(struct spa_pod_builder *builder, uint32_
 
 #define SPA_POD_INIT_Bool(val) ((struct spa_pod_bool){ { sizeof(uint32_t), SPA_TYPE_Bool }, (val) ? 1 : 0, 0 })
 
-static inline int spa_pod_builder_bool(struct spa_pod_builder *builder, bool val)
+SPA_API_POD_BUILDER int spa_pod_builder_bool(struct spa_pod_builder *builder, bool val)
 {
 	const struct spa_pod_bool p = SPA_POD_INIT_Bool(val);
 	return spa_pod_builder_primitive(builder, &p.pod);
@@ -221,7 +238,7 @@ static inline int spa_pod_builder_bool(struct spa_pod_builder *builder, bool val
 
 #define SPA_POD_INIT_Id(val) ((struct spa_pod_id){ { sizeof(uint32_t), SPA_TYPE_Id }, (val), 0 })
 
-static inline int spa_pod_builder_id(struct spa_pod_builder *builder, uint32_t val)
+SPA_API_POD_BUILDER int spa_pod_builder_id(struct spa_pod_builder *builder, uint32_t val)
 {
 	const struct spa_pod_id p = SPA_POD_INIT_Id(val);
 	return spa_pod_builder_primitive(builder, &p.pod);
@@ -229,7 +246,7 @@ static inline int spa_pod_builder_id(struct spa_pod_builder *builder, uint32_t v
 
 #define SPA_POD_INIT_Int(val) ((struct spa_pod_int){ { sizeof(int32_t), SPA_TYPE_Int }, (val), 0 })
 
-static inline int spa_pod_builder_int(struct spa_pod_builder *builder, int32_t val)
+SPA_API_POD_BUILDER int spa_pod_builder_int(struct spa_pod_builder *builder, int32_t val)
 {
 	const struct spa_pod_int p = SPA_POD_INIT_Int(val);
 	return spa_pod_builder_primitive(builder, &p.pod);
@@ -237,7 +254,7 @@ static inline int spa_pod_builder_int(struct spa_pod_builder *builder, int32_t v
 
 #define SPA_POD_INIT_Long(val) ((struct spa_pod_long){ { sizeof(int64_t), SPA_TYPE_Long }, (val) })
 
-static inline int spa_pod_builder_long(struct spa_pod_builder *builder, int64_t val)
+SPA_API_POD_BUILDER int spa_pod_builder_long(struct spa_pod_builder *builder, int64_t val)
 {
 	const struct spa_pod_long p = SPA_POD_INIT_Long(val);
 	return spa_pod_builder_primitive(builder, &p.pod);
@@ -245,7 +262,7 @@ static inline int spa_pod_builder_long(struct spa_pod_builder *builder, int64_t
 
 #define SPA_POD_INIT_Float(val) ((struct spa_pod_float){ { sizeof(float), SPA_TYPE_Float }, (val), 0 })
 
-static inline int spa_pod_builder_float(struct spa_pod_builder *builder, float val)
+SPA_API_POD_BUILDER int spa_pod_builder_float(struct spa_pod_builder *builder, float val)
 {
 	const struct spa_pod_float p = SPA_POD_INIT_Float(val);
 	return spa_pod_builder_primitive(builder, &p.pod);
@@ -253,7 +270,7 @@ static inline int spa_pod_builder_float(struct spa_pod_builder *builder, float v
 
 #define SPA_POD_INIT_Double(val) ((struct spa_pod_double){ { sizeof(double), SPA_TYPE_Double }, (val) })
 
-static inline int spa_pod_builder_double(struct spa_pod_builder *builder, double val)
+SPA_API_POD_BUILDER int spa_pod_builder_double(struct spa_pod_builder *builder, double val)
 {
 	const struct spa_pod_double p = SPA_POD_INIT_Double(val);
 	return spa_pod_builder_primitive(builder, &p.pod);
@@ -261,7 +278,7 @@ static inline int spa_pod_builder_double(struct spa_pod_builder *builder, double
 
 #define SPA_POD_INIT_String(len) ((struct spa_pod_string){ { (len), SPA_TYPE_String } })
 
-static inline int
+SPA_API_POD_BUILDER int
 spa_pod_builder_write_string(struct spa_pod_builder *builder, const char *str, uint32_t len)
 {
 	int r, res;
@@ -273,7 +290,7 @@ spa_pod_builder_write_string(struct spa_pod_builder *builder, const char *str, u
 	return res;
 }
 
-static inline int
+SPA_API_POD_BUILDER int
 spa_pod_builder_string_len(struct spa_pod_builder *builder, const char *str, uint32_t len)
 {
 	const struct spa_pod_string p = SPA_POD_INIT_String(len+1);
@@ -283,7 +300,7 @@ spa_pod_builder_string_len(struct spa_pod_builder *builder, const char *str, uin
 	return res;
 }
 
-static inline int spa_pod_builder_string(struct spa_pod_builder *builder, const char *str)
+SPA_API_POD_BUILDER int spa_pod_builder_string(struct spa_pod_builder *builder, const char *str)
 {
 	uint32_t len = str ? strlen(str) : 0;
 	return spa_pod_builder_string_len(builder, str ? str : "", len);
@@ -291,7 +308,7 @@ static inline int spa_pod_builder_string(struct spa_pod_builder *builder, const
 
 #define SPA_POD_INIT_Bytes(len) ((struct spa_pod_bytes){ { (len), SPA_TYPE_Bytes } })
 
-static inline int
+SPA_API_POD_BUILDER int
 spa_pod_builder_bytes(struct spa_pod_builder *builder, const void *bytes, uint32_t len)
 {
 	const struct spa_pod_bytes p = SPA_POD_INIT_Bytes(len);
@@ -300,7 +317,7 @@ spa_pod_builder_bytes(struct spa_pod_builder *builder, const void *bytes, uint32
 		res = r;
 	return res;
 }
-static inline void *
+SPA_API_POD_BUILDER void *
 spa_pod_builder_reserve_bytes(struct spa_pod_builder *builder, uint32_t len)
 {
 	uint32_t offset = builder->state.offset;
@@ -309,9 +326,34 @@ spa_pod_builder_reserve_bytes(struct spa_pod_builder *builder, uint32_t len)
 	return SPA_POD_BODY(spa_pod_builder_deref(builder, offset));
 }
 
+SPA_API_POD_BUILDER uint32_t
+spa_pod_builder_bytes_start(struct spa_pod_builder *builder)
+{
+	uint32_t offset = builder->state.offset;
+	const struct spa_pod_bytes p = SPA_POD_INIT_Bytes(0);
+	spa_pod_builder_raw(builder, &p, sizeof(p));
+	return offset;
+}
+SPA_API_POD_BUILDER int
+spa_pod_builder_bytes_append(struct spa_pod_builder *builder, uint32_t offset,
+		const void *data, uint32_t size)
+{
+	int res = spa_pod_builder_raw(builder, data, size);
+	struct spa_pod *pod = spa_pod_builder_deref(builder, offset);
+	if (pod)
+		pod->size += size;
+	return res;
+}
+
+SPA_API_POD_BUILDER int
+spa_pod_builder_bytes_end(struct spa_pod_builder *builder, uint32_t offset SPA_UNUSED)
+{
+	return spa_pod_builder_pad(builder, builder->state.offset);
+}
+
 #define SPA_POD_INIT_Pointer(type,value) ((struct spa_pod_pointer){ { sizeof(struct spa_pod_pointer_body), SPA_TYPE_Pointer }, { (type), 0, (value) } })
 
-static inline int
+SPA_API_POD_BUILDER int
 spa_pod_builder_pointer(struct spa_pod_builder *builder, uint32_t type, const void *val)
 {
 	const struct spa_pod_pointer p = SPA_POD_INIT_Pointer(type, val);
@@ -320,7 +362,7 @@ spa_pod_builder_pointer(struct spa_pod_builder *builder, uint32_t type, const vo
 
 #define SPA_POD_INIT_Fd(fd) ((struct spa_pod_fd){ { sizeof(int64_t), SPA_TYPE_Fd }, (fd) })
 
-static inline int spa_pod_builder_fd(struct spa_pod_builder *builder, int64_t fd)
+SPA_API_POD_BUILDER int spa_pod_builder_fd(struct spa_pod_builder *builder, int64_t fd)
 {
 	const struct spa_pod_fd p = SPA_POD_INIT_Fd(fd);
 	return spa_pod_builder_primitive(builder, &p.pod);
@@ -328,7 +370,7 @@ static inline int spa_pod_builder_fd(struct spa_pod_builder *builder, int64_t fd
 
 #define SPA_POD_INIT_Rectangle(val) ((struct spa_pod_rectangle){ { sizeof(struct spa_rectangle), SPA_TYPE_Rectangle }, (val) })
 
-static inline int
+SPA_API_POD_BUILDER int
 spa_pod_builder_rectangle(struct spa_pod_builder *builder, uint32_t width, uint32_t height)
 {
 	const struct spa_pod_rectangle p = SPA_POD_INIT_Rectangle(SPA_RECTANGLE(width, height));
@@ -337,14 +379,14 @@ spa_pod_builder_rectangle(struct spa_pod_builder *builder, uint32_t width, uint3
 
 #define SPA_POD_INIT_Fraction(val) ((struct spa_pod_fraction){ { sizeof(struct spa_fraction), SPA_TYPE_Fraction }, (val) })
 
-static inline int
+SPA_API_POD_BUILDER int
 spa_pod_builder_fraction(struct spa_pod_builder *builder, uint32_t num, uint32_t denom)
 {
 	const struct spa_pod_fraction p = SPA_POD_INIT_Fraction(SPA_FRACTION(num, denom));
 	return spa_pod_builder_primitive(builder, &p.pod);
 }
 
-static inline int
+SPA_API_POD_BUILDER int
 spa_pod_builder_push_array(struct spa_pod_builder *builder, struct spa_pod_frame *frame)
 {
 	const struct spa_pod_array p =
@@ -356,7 +398,7 @@ spa_pod_builder_push_array(struct spa_pod_builder *builder, struct spa_pod_frame
 	return res;
 }
 
-static inline int
+SPA_API_POD_BUILDER int
 spa_pod_builder_array(struct spa_pod_builder *builder,
 		      uint32_t child_size, uint32_t child_type, uint32_t n_elems, const void *elems)
 {
@@ -378,7 +420,7 @@ spa_pod_builder_array(struct spa_pod_builder *builder,
 	{ { { (n_vals) * sizeof(ctype) + sizeof(struct spa_pod_choice_body), SPA_TYPE_Choice },	\
 		{ (type), 0, { sizeof(ctype), (child_type) } } }, { __VA_ARGS__ } })
 
-static inline int
+SPA_API_POD_BUILDER int
 spa_pod_builder_push_choice(struct spa_pod_builder *builder, struct spa_pod_frame *frame,
 		uint32_t type, uint32_t flags)
 {
@@ -393,7 +435,7 @@ spa_pod_builder_push_choice(struct spa_pod_builder *builder, struct spa_pod_fram
 
 #define SPA_POD_INIT_Struct(size) ((struct spa_pod_struct){ { (size), SPA_TYPE_Struct } })
 
-static inline int
+SPA_API_POD_BUILDER int
 spa_pod_builder_push_struct(struct spa_pod_builder *builder, struct spa_pod_frame *frame)
 {
 	const struct spa_pod_struct p = SPA_POD_INIT_Struct(0);
@@ -405,7 +447,7 @@ spa_pod_builder_push_struct(struct spa_pod_builder *builder, struct spa_pod_fram
 
 #define SPA_POD_INIT_Object(size,type,id,...)	((struct spa_pod_object){ { (size), SPA_TYPE_Object }, { (type), (id) }, ##__VA_ARGS__ })
 
-static inline int
+SPA_API_POD_BUILDER int
 spa_pod_builder_push_object(struct spa_pod_builder *builder, struct spa_pod_frame *frame,
 		uint32_t type, uint32_t id)
 {
@@ -420,7 +462,7 @@ spa_pod_builder_push_object(struct spa_pod_builder *builder, struct spa_pod_fram
 #define SPA_POD_INIT_Prop(key,flags,size,type)	\
 	((struct spa_pod_prop){ (key), (flags), { (size), (type) } })
 
-static inline int
+SPA_API_POD_BUILDER int
 spa_pod_builder_prop(struct spa_pod_builder *builder, uint32_t key, uint32_t flags)
 {
 	const struct { uint32_t key; uint32_t flags; } p = { key, flags };
@@ -430,7 +472,7 @@ spa_pod_builder_prop(struct spa_pod_builder *builder, uint32_t key, uint32_t fla
 #define SPA_POD_INIT_Sequence(size,unit)	\
 	((struct spa_pod_sequence){ { (size), SPA_TYPE_Sequence}, {(unit), 0 } })
 
-static inline int
+SPA_API_POD_BUILDER int
 spa_pod_builder_push_sequence(struct spa_pod_builder *builder, struct spa_pod_frame *frame, uint32_t unit)
 {
 	const struct spa_pod_sequence p =
@@ -441,14 +483,14 @@ spa_pod_builder_push_sequence(struct spa_pod_builder *builder, struct spa_pod_fr
 	return res;
 }
 
-static inline int
+SPA_API_POD_BUILDER int
 spa_pod_builder_control(struct spa_pod_builder *builder, uint32_t offset, uint32_t type)
 {
 	const struct { uint32_t offset; uint32_t type; } p = { offset, type };
 	return spa_pod_builder_raw(builder, &p, sizeof(p));
 }
 
-static inline uint32_t spa_choice_from_id(char id)
+SPA_API_POD_BUILDER uint32_t spa_choice_from_id(char id)
 {
 	switch (id) {
 	case 'r':
@@ -560,7 +602,7 @@ do {										\
 	}									\
 } while(false)
 
-static inline int
+SPA_API_POD_BUILDER int
 spa_pod_builder_addv(struct spa_pod_builder *builder, va_list args)
 {
 	int res = 0;
@@ -618,7 +660,7 @@ spa_pod_builder_addv(struct spa_pod_builder *builder, va_list args)
 	return res;
 }
 
-static inline int spa_pod_builder_add(struct spa_pod_builder *builder, ...)
+SPA_API_POD_BUILDER int spa_pod_builder_add(struct spa_pod_builder *builder, ...)
 {
 	int res;
 	va_list args;
@@ -658,7 +700,7 @@ static inline int spa_pod_builder_add(struct spa_pod_builder *builder, ...)
 })
 
 /** Copy a pod structure */
-static inline struct spa_pod *
+SPA_API_POD_BUILDER struct spa_pod *
 spa_pod_copy(const struct spa_pod *pod)
 {
 	size_t size;
