@@ -30,7 +30,7 @@ struct spa_event {
 					(ev)->body.body.id : SPA_ID_INVALID)
 
 #define SPA_EVENT_INIT_FULL(t,size,type,id,...) ((t)			\
-	{ { (size), SPA_TYPE_OBJECT },					\
+	{ { (size), SPA_TYPE_Object },					\
 	  { { (type), (id) }, ##__VA_ARGS__ } })			\
 
 #define SPA_EVENT_INIT(type,id)						\
