@@ -28,6 +28,7 @@ static void reset_props(struct props *props)
 {
 	strncpy(props->device, default_device, 64);
 	props->use_chmap = DEFAULT_USE_CHMAP;
+	spa_scnprintf(props->media_class, sizeof(props->media_class), "%s", "Audio/Source");
 }
 
 static int impl_node_enum_params(void *object, int seq,
@@ -292,11 +293,13 @@ static int impl_node_send_command(void *object, const struct spa_command *comman
 		if (this->n_buffers == 0)
 			return -EIO;
 
+		this->want_started = true;
 		if ((res = spa_alsa_start(this)) < 0)
 			return res;
 		break;
 	case SPA_NODE_COMMAND_Pause:
 	case SPA_NODE_COMMAND_Suspend:
+		this->want_started = false;
 		if ((res = spa_alsa_pause(this)) < 0)
 			return res;
 		break;
@@ -685,7 +688,8 @@ impl_node_port_set_io(void *object,
 		break;
 	case SPA_IO_RateMatch:
 		this->rate_match = data;
-		spa_alsa_update_rate_match(this);
+		if (this->rate_match)
+			spa_alsa_update_rate_match(this);
 		break;
 	default:
 		return -ENOENT;
