@@ -23,8 +23,8 @@
 #include "mix-ops.h"
 
 #undef SPA_LOG_TOPIC_DEFAULT
-#define SPA_LOG_TOPIC_DEFAULT log_topic
-static struct spa_log_topic *log_topic = &SPA_LOG_TOPIC(0, "spa.mixer-dsp");
+#define SPA_LOG_TOPIC_DEFAULT &log_topic
+SPA_LOG_TOPIC_DEFINE_STATIC(log_topic, "spa.mixer-dsp");
 
 #define MAX_BUFFERS	64
 #define MAX_PORTS	512
@@ -61,7 +61,7 @@ struct port {
 
 	struct port_props props;
 
-	struct spa_io_buffers *io;
+	struct spa_io_buffers *io[2];
 
 	uint64_t info_all;
 	struct spa_port_info info;
@@ -96,6 +96,8 @@ struct impl {
 	struct spa_node_info info;
 	struct spa_param_info params[8];
 
+	struct spa_io_position *position;
+
 	struct spa_hook_list hooks;
 
 	uint32_t port_count;
@@ -115,13 +117,16 @@ struct impl {
 };
 
 #define PORT_VALID(p)                ((p) != NULL && (p)->valid)
+#define CHECK_ANY_IN(this,d,p)       ((d) == SPA_DIRECTION_INPUT && (p) == SPA_ID_INVALID)
 #define CHECK_FREE_IN_PORT(this,d,p) ((d) == SPA_DIRECTION_INPUT && (p) < MAX_PORTS && !PORT_VALID(this->in_ports[(p)]))
 #define CHECK_IN_PORT(this,d,p)      ((d) == SPA_DIRECTION_INPUT && (p) < MAX_PORTS && PORT_VALID(this->in_ports[(p)]))
 #define CHECK_OUT_PORT(this,d,p)     ((d) == SPA_DIRECTION_OUTPUT && (p) == 0)
 #define CHECK_PORT(this,d,p)         (CHECK_OUT_PORT(this,d,p) || CHECK_IN_PORT (this,d,p))
+#define CHECK_PORT_ANY(this,d,p)     (CHECK_ANY_IN(this,d,p) || CHECK_PORT(this,d,p))
 #define GET_IN_PORT(this,p)          (this->in_ports[p])
 #define GET_OUT_PORT(this,p)         (&this->out_ports[p])
 #define GET_PORT(this,d,p)           (d == SPA_DIRECTION_INPUT ? GET_IN_PORT(this,p) : GET_OUT_PORT(this,p))
+#define GET_PORT_ANY(this,d,p)       (CHECK_ANY_IN(this,d,p) ? NULL : GET_PORT(this,d,p))
 
 static int impl_node_enum_params(void *object, int seq,
 				 uint32_t id, uint32_t start, uint32_t num,
@@ -138,7 +143,16 @@ static int impl_node_set_param(void *object, uint32_t id, uint32_t flags,
 
 static int impl_node_set_io(void *object, uint32_t id, void *data, size_t size)
 {
-	return -ENOTSUP;
+	struct impl *this = object;
+
+	switch (id) {
+	case SPA_IO_Position:
+		this->position = data;
+		break;
+	default:
+		return -ENOTSUP;
+	}
+	return 0;
 }
 
 static int impl_node_send_command(void *object, const struct spa_command *command)
@@ -303,8 +317,7 @@ impl_node_remove_port(void *object, enum spa_direction direction, uint32_t port_
 	return 0;
 }
 
-static int port_enum_formats(void *object,
-			     enum spa_direction direction, uint32_t port_id,
+static int port_enum_formats(void *object, struct port *port,
 			     uint32_t index,
 			     struct spa_pod **param,
 			     struct spa_pod_builder *builder)
@@ -347,9 +360,9 @@ impl_node_port_enum_params(void *object, int seq,
 
 	spa_return_val_if_fail(this != NULL, -EINVAL);
 	spa_return_val_if_fail(num != 0, -EINVAL);
-	spa_return_val_if_fail(CHECK_PORT(this, direction, port_id), -EINVAL);
+	spa_return_val_if_fail(CHECK_PORT_ANY(this, direction, port_id), -EINVAL);
 
-	port = GET_PORT(this, direction, port_id);
+	port = GET_PORT_ANY(this, direction, port_id);
 
 	result.id = id;
 	result.next = start;
@@ -360,12 +373,12 @@ next:
 
 	switch (id) {
 	case SPA_PARAM_EnumFormat:
-		if ((res = port_enum_formats(this, direction, port_id, result.index, &param, &b)) <= 0)
+		if ((res = port_enum_formats(this, port, result.index, &param, &b)) <= 0)
 			return res;
 		break;
 
 	case SPA_PARAM_Format:
-		if (!port->have_format)
+		if (port == NULL || !port->have_format)
 			return -EIO;
 		if (result.index > 0)
 			return 0;
@@ -374,7 +387,7 @@ next:
 		break;
 
 	case SPA_PARAM_Buffers:
-		if (!port->have_format)
+		if (port == NULL || !port->have_format)
 			return -EIO;
 		if (result.index > 0)
 			return 0;
@@ -411,6 +424,12 @@ next:
 				SPA_PARAM_IO_id,   SPA_POD_Id(SPA_IO_Buffers),
 				SPA_PARAM_IO_size, SPA_POD_Int(sizeof(struct spa_io_buffers)));
 			break;
+		case 1:
+			param = spa_pod_builder_add_object(&b,
+				SPA_TYPE_OBJECT_ParamIO, id,
+				SPA_PARAM_IO_id,   SPA_POD_Id(SPA_IO_AsyncBuffers),
+				SPA_PARAM_IO_size, SPA_POD_Int(sizeof(struct spa_io_async_buffers)));
+			break;
 		default:
 			return 0;
 		}
@@ -616,13 +635,24 @@ impl_node_port_use_buffers(void *object,
 struct io_info {
 	struct port *port;
 	void *data;
+	size_t size;
 };
 
 static int do_port_set_io(struct spa_loop *loop, bool async, uint32_t seq,
 		const void *data, size_t size, void *user_data)
 {
 	struct io_info *info = user_data;
-	info->port->io = info->data;
+	if (info->size >= sizeof(struct spa_io_async_buffers)) {
+		struct spa_io_async_buffers *ab = info->data;
+		info->port->io[0] = &ab->buffers[info->port->direction];
+		info->port->io[1] = &ab->buffers[info->port->direction^1];
+	} else if (info->size >= sizeof(struct spa_io_buffers)) {
+		info->port->io[0] = info->data;
+		info->port->io[1] = info->data;
+	} else {
+		info->port->io[0] = NULL;
+		info->port->io[1] = NULL;
+	}
 	return 0;
 }
 
@@ -645,9 +675,11 @@ impl_node_port_set_io(void *object,
 	port = GET_PORT(this, direction, port_id);
 	info.port = port;
 	info.data = data;
+	info.size = size;
 
 	switch (id) {
 	case SPA_IO_Buffers:
+	case SPA_IO_AsyncBuffers:
 		spa_loop_invoke(this->data_loop,
                                do_port_set_io, SPA_ID_INVALID, NULL, 0, true, &info);
 		break;
@@ -681,11 +713,12 @@ static int impl_node_process(void *object)
 	struct buffer **buffers;
 	struct buffer *outb;
 	const void **datas;
+	uint32_t cycle = this->position->clock.cycle & 1;
 
 	spa_return_val_if_fail(this != NULL, -EINVAL);
 
 	outport = GET_OUT_PORT(this, 0);
-	if ((outio = outport->io) == NULL)
+	if ((outio = outport->io[cycle]) == NULL)
 		return -EIO;
 
 	spa_log_trace_fp(this->log, "%p: status %p %d %d",
@@ -713,16 +746,17 @@ static int impl_node_process(void *object)
 		struct spa_data *bd;
 		uint32_t size, offs;
 
-		if (SPA_UNLIKELY(!PORT_VALID(inport) ||
-		    (inio = inport->io) == NULL ||
-		    inio->buffer_id >= inport->n_buffers ||
-		    inio->status != SPA_STATUS_HAVE_DATA)) {
-			spa_log_trace_fp(this->log, "%p: skip input idx:%d valid:%d "
+		if (SPA_UNLIKELY(!PORT_VALID(inport) || (inio = inport->io[cycle]) == NULL)) {
+			spa_log_trace_fp(this->log, "%p: skip input idx:%d valid:%d io:%p/%p/%d",
+					this, i, PORT_VALID(inport),
+					inport->io[0], inport->io[1], cycle);
+			continue;
+		}
+		if (inio->buffer_id >= inport->n_buffers ||
+		    inio->status != SPA_STATUS_HAVE_DATA) {
+			spa_log_trace_fp(this->log, "%p: skip input idx:%d "
 					"io:%p status:%d buf_id:%d n_buffers:%d", this,
-				i, PORT_VALID(inport), inio,
-				inio ? inio->status : -1,
-				inio ? inio->buffer_id : SPA_ID_INVALID,
-				inport->n_buffers);
+				i, inio, inio->status, inio->buffer_id, inport->n_buffers);
 			continue;
 		}
 
@@ -733,8 +767,8 @@ static int impl_node_process(void *object)
 		size = SPA_MIN(bd->maxsize - offs, bd->chunk->size);
 		maxsize = SPA_MIN(maxsize, size);
 
-		spa_log_trace_fp(this->log, "%p: mix input %d %p->%p %d %d %d:%d/%d %u", this,
-				i, inio, outio, inio->status, inio->buffer_id,
+		spa_log_trace_fp(this->log, "%p: mix input %d %p->%p %d %d/%d %d:%d/%d %u", this,
+				i, inio, outio, inio->status, inio->buffer_id, inport->n_buffers,
 				offs, size, (int)sizeof(float),
 				bd->chunk->flags);
 
@@ -855,7 +889,7 @@ impl_init(const struct spa_handle_factory *factory,
 	this = (struct impl *) handle;
 
 	this->log = spa_support_find(support, n_support, SPA_TYPE_INTERFACE_Log);
-	spa_log_topic_init(this->log, log_topic);
+	spa_log_topic_init(this->log, &log_topic);
 
 	this->data_loop = spa_support_find(support, n_support, SPA_TYPE_INTERFACE_DataLoop);
 	if (this->data_loop == NULL) {
