@@ -44,6 +44,13 @@ struct _GstPipeWirePool {
   GstAllocator *dmabuf_allocator;
 
   GCond cond;
+  gboolean paused;
+};
+
+enum GstPipeWirePoolMode {
+    USE_BUFFERPOOL_NO = 0,
+    USE_BUFFERPOOL_AUTO,
+    USE_BUFFERPOOL_YES
 };
 
 GstPipeWirePool *  gst_pipewire_pool_new (GstPipeWireStream *stream);
@@ -59,6 +66,8 @@ gst_pipewire_pool_has_buffers (GstPipeWirePool *pool)
 
 GstPipeWirePoolData *gst_pipewire_pool_get_data (GstBuffer *buffer);
 
+void gst_pipewire_pool_set_paused (GstPipeWirePool *pool, gboolean paused);
+
 G_END_DECLS
 
 #endif /* __GST_PIPEWIRE_POOL_H__ */
