@@ -20,12 +20,12 @@
 #include <sys/mount.h>
 #endif
 
+#include <spa/utils/cleanup.h>
 #include <spa/utils/result.h>
 #include <spa/utils/string.h>
 #include <spa/utils/json.h>
 
 #include <pipewire/impl.h>
-#include <pipewire/cleanup.h>
 
 #include "flatpak-utils.h"
 
@@ -95,6 +95,22 @@
  * - \ref PW_KEY_ACCESS
  * - \ref PW_KEY_CLIENT_ACCESS
  *
+ * ## Config override
+ *
+ * A `module.access.args` config section can be added
+ * to override the module arguments.
+ *
+ *\code{.unparsed}
+ * # ~/.config/pipewire/pipewire.conf.d/my-access-args.conf
+ *
+ * module.access.args = {
+ *      access.socket = {
+ *          pipewire-0 = "default",
+ *          pipewire-0-manager = "unrestricted",
+ *      }
+ * }
+ *\endcode
+ *
  * ## Example configuration
  *
  *\code{.unparsed}
@@ -274,21 +290,16 @@ get_server_name(const struct spa_dict *props)
 
 static int parse_socket_args(struct impl *impl, const char *str)
 {
-	struct spa_json it[2];
+	struct spa_json it[1];
 	char socket[PATH_MAX];
+	const char *val;
+	int len;
 
-	spa_json_init(&it[0], str, strlen(str));
-
-	if (spa_json_enter_object(&it[0], &it[1]) <= 0)
+	if (spa_json_begin_object(&it[0], str, strlen(str)) <= 0)
 		return -EINVAL;
 
-	while (spa_json_get_string(&it[1], socket, sizeof(socket)) > 0) {
+	while ((len = spa_json_object_next(&it[0], socket, sizeof(socket), &val)) > 0) {
 		char value[256];
-		const char *val;
-		int len;
-
-		if ((len = spa_json_next(&it[1], &val)) <= 0)
-			return -EINVAL;
 
 		if (spa_json_parse_stringn(val, len, value, sizeof(value)) <= 0)
 			return -EINVAL;
@@ -299,17 +310,11 @@ static int parse_socket_args(struct impl *impl, const char *str)
 	return 0;
 }
 
-static int parse_args(struct impl *impl, const struct pw_properties *props, const char *args_str)
+static int parse_args(struct impl *impl, const struct pw_properties *props, const struct pw_properties *args)
 {
-	spa_autoptr(pw_properties) args = NULL;
 	const char *str;
 	int res;
 
-	if (args_str)
-		args = pw_properties_new_string(args_str);
-	else
-		args = pw_properties_new(NULL, NULL);
-
 	if ((str = pw_properties_get(args, "access.legacy")) != NULL) {
 		impl->legacy = spa_atob(str);
 	} else if (pw_properties_get(args, "access.socket")) {
@@ -352,10 +357,11 @@ static int parse_args(struct impl *impl, const struct pw_properties *props, cons
 }
 
 SPA_EXPORT
-int pipewire__module_init(struct pw_impl_module *module, const char *args)
+int pipewire__module_init(struct pw_impl_module *module, const char *args_str)
 {
 	struct pw_context *context = pw_impl_module_get_context(module);
 	const struct pw_properties *props = pw_context_get_properties(context);
+	spa_autoptr(pw_properties) args = NULL;
 	struct impl *impl;
 	int res;
 
@@ -365,7 +371,19 @@ int pipewire__module_init(struct pw_impl_module *module, const char *args)
 	if (impl == NULL)
 		return -errno;
 
-	pw_log_debug("module %p: new %s", impl, args);
+	pw_log_debug("module %p: new %s", impl, args_str);
+
+	if (args_str)
+		args = pw_properties_new_string(args_str);
+	else
+		args = pw_properties_new(NULL, NULL);
+
+	if (!args) {
+		res = -errno;
+		goto error;
+	}
+
+	pw_context_conf_update_props(context, "module."NAME".args", args);
 
 	impl->socket_access = pw_properties_new(NULL, NULL);
 
