@@ -242,17 +242,78 @@ static inline void fix_midi_event(uint8_t *data, size_t size)
 	}
 }
 
+static inline void *n2j_midi_buffer_reserve(struct nj2_midi_buffer *buf,
+		uint32_t offset, uint32_t size)
+{
+	struct nj2_midi_event *ev;
+	void *ptr;
+
+	if (size <= 0)
+		return NULL;
+
+	size_t used_size = sizeof(*buf) + buf->write_pos +
+		((buf->event_count + 1) * sizeof(struct nj2_midi_event));
+
+	ev = &buf->event[buf->event_count];
+	ev->time = offset;
+	ev->size = size;
+	if (size <= MIDI_INLINE_MAX) {
+		ptr = ev->buffer;
+	} else {
+		if (used_size + size > buf->buffer_size)
+			return NULL;
+		buf->write_pos += size;
+		ev->offset = buf->buffer_size - buf->write_pos;
+		ptr = SPA_PTROFF(buf, ev->offset, void);
+	}
+	buf->event_count++;
+	return ptr;
+}
+
+static inline void n2j_midi_buffer_write(struct nj2_midi_buffer *buf,
+		uint32_t offset, void *data, uint32_t size)
+{
+	void *ptr = n2j_midi_buffer_reserve(buf, offset, size);
+	if (ptr != NULL)
+		memcpy(ptr, data, size);
+	else
+		buf->lost_events++;
+}
+
+static inline void n2j_midi_buffer_append(struct nj2_midi_buffer *buf,
+		void *data, uint32_t size)
+{
+	struct nj2_midi_event *ev;
+	uint32_t old_size;
+	uint8_t *old_ptr, *new_ptr;
+
+	ev = &buf->event[--buf->event_count];
+	old_size = ev->size;
+	if (old_size <= MIDI_INLINE_MAX) {
+		old_ptr = ev->buffer;
+	} else {
+		buf->write_pos -= old_size;
+		old_ptr = SPA_PTROFF(buf, ev->offset, void);
+	}
+	new_ptr = n2j_midi_buffer_reserve(buf, ev->time, old_size + size);
+	if (new_ptr == NULL) {
+		buf->lost_events++;
+	} else {
+		memmove(new_ptr, old_ptr, old_size);
+		memcpy(new_ptr+old_size, data, size);
+	}
+}
+
 static void midi_to_netjack2(struct netjack2_peer *peer,
 		struct nj2_midi_buffer *buf, float *src, uint32_t n_samples)
 {
 	struct spa_pod *pod;
 	struct spa_pod_sequence *seq;
 	struct spa_pod_control *c;
-	struct nj2_midi_event *ev;
-	int free_size;
+	bool in_sysex = false;
 
 	buf->magic = MIDI_BUFFER_MAGIC;
-	buf->buffer_size = peer->quantum_limit * sizeof(float);
+	buf->buffer_size = peer->params.period_size * sizeof(float);
 	buf->nframes = n_samples;
 	buf->write_pos = 0;
 	buf->event_count = 0;
@@ -269,12 +330,10 @@ static void midi_to_netjack2(struct netjack2_peer *peer,
 
 	seq = (struct spa_pod_sequence*)pod;
 
-	free_size = buf->buffer_size - sizeof(*buf);
-
 	SPA_POD_SEQUENCE_FOREACH(seq, c) {
 		int size;
 		uint8_t data[16];
-		void *ptr;
+		bool was_sysex = in_sysex;
 
 		if (c->type != SPA_CONTROL_UMP)
 			continue;
@@ -284,29 +343,24 @@ static void midi_to_netjack2(struct netjack2_peer *peer,
 		if (size <= 0)
 			continue;
 
-		if (c->offset >= n_samples ||
-		    size >= free_size) {
+		if (c->offset >= n_samples) {
 			buf->lost_events++;
 			continue;
 		}
 
-		if (peer->fix_midi)
+		if (!in_sysex && data[0] == 0xf0)
+			in_sysex = true;
+
+		if (!in_sysex && peer->fix_midi)
 			fix_midi_event(data, size);
 
-		ev = &buf->event[buf->event_count];
-		ev->time = c->offset;
-		ev->size = size;
-		if (size <= MIDI_INLINE_MAX) {
-			ptr = ev->buffer;
-		} else {
-			buf->write_pos += size;
-			ev->offset = buf->buffer_size - 1 - buf->write_pos;
-			free_size -= size;
-			ptr = SPA_PTROFF(buf, ev->offset, void);
-		}
-		memcpy(ptr, data, size);
-		buf->event_count++;
-		free_size -= sizeof(*ev);
+		if (in_sysex && data[size-1] == 0xf7)
+			in_sysex = false;
+
+		if (was_sysex)
+			n2j_midi_buffer_append(buf, data, size);
+		else
+			n2j_midi_buffer_write(buf, c->offset, data, size);
 	}
 	if (buf->write_pos > 0)
 		memmove(SPA_PTROFF(buf, sizeof(*buf) + buf->event_count * sizeof(struct nj2_midi_event), void),
@@ -314,15 +368,27 @@ static void midi_to_netjack2(struct netjack2_peer *peer,
 			buf->write_pos);
 }
 
+static inline void netjack2_clear_midi(float *dst, uint32_t size)
+{
+	struct spa_pod_builder b = { 0, };
+	struct spa_pod_frame f;
+	spa_pod_builder_init(&b, dst, size);
+	spa_pod_builder_push_sequence(&b, &f, 0);
+	spa_pod_builder_pop(&b, &f);
+}
+
 static inline void netjack2_to_midi(float *dst, uint32_t size, struct nj2_midi_buffer *buf)
 {
 	struct spa_pod_builder b = { 0, };
 	uint32_t i;
 	struct spa_pod_frame f;
+	size_t offset = size - buf->write_pos - sizeof(*buf) -
+			(buf->event_count * sizeof(struct nj2_midi_event));
 
 	spa_pod_builder_init(&b, dst, size);
 	spa_pod_builder_push_sequence(&b, &f, 0);
-	for (i = 0; buf != NULL && i < buf->event_count; i++) {
+
+	for (i = 0; i < buf->event_count; i++) {
 		struct nj2_midi_event *ev = &buf->event[i];
 		uint8_t *data;
 		size_t s;
@@ -330,8 +396,8 @@ static inline void netjack2_to_midi(float *dst, uint32_t size, struct nj2_midi_b
 
 		if (ev->size <= MIDI_INLINE_MAX)
 			data = ev->buffer;
-		else if (ev->offset > buf->write_pos)
-			data = SPA_PTROFF(buf, ev->offset - buf->write_pos, void);
+		else if (ev->offset > offset)
+			data = SPA_PTROFF(buf, ev->offset - offset, void);
 		else
 			continue;
 
@@ -339,9 +405,10 @@ static inline void netjack2_to_midi(float *dst, uint32_t size, struct nj2_midi_b
 		while (s > 0) {
 			uint32_t ump[4];
 			int ump_size = spa_ump_from_midi(&data, &s, ump, sizeof(ump), 0, &state);
-			if (ump_size <= 0)
+			if (ump_size <= 0) {
+				pw_log_warn("invalid MIDI received: %s", spa_strerror(ump_size));
 				break;
-
+			}
 			spa_pod_builder_control(&b, ev->time, SPA_CONTROL_UMP);
 	                spa_pod_builder_bytes(&b, ump, ump_size);
 		}
@@ -362,7 +429,7 @@ static int netjack2_send_sync(struct netjack2_peer *peer, uint32_t nframes)
 	is_last = peer->params.send_midi_channels == 0 &&
                         peer->params.send_audio_channels == 0 ? 1 : 0;
 
-	strcpy(header.type, "header");
+	strncpy(header.type, "header", sizeof(header.type));
 	header.data_type = htonl('s');
 	header.data_stream = htonl(peer->our_stream);
 	header.id = htonl(peer->params.id);
@@ -416,7 +483,7 @@ static int netjack2_send_midi(struct netjack2_peer *peer, uint32_t nframes,
 	max_size = peer->params.mtu - sizeof(header);
 	num_packets = (midi_size + max_size-1) / max_size;
 
-	strcpy(header.type, "header");
+	strncpy(header.type, "header", sizeof(header.type));
 	header.data_type = htonl('m');
 	header.data_stream = htonl(peer->our_stream);
 	header.id = htonl(peer->params.id);
@@ -468,7 +535,7 @@ static int netjack2_send_float(struct netjack2_peer *peer, uint32_t nframes,
 	sub_period_bytes = sub_period_size * sizeof(float) + sizeof(int32_t);
 	num_packets = nframes / sub_period_size;
 
-	strcpy(header.type, "header");
+	strncpy(header.type, "header", sizeof(header.type));
 	header.data_type = htonl('a');
 	header.data_stream = htonl(peer->our_stream);
 	header.id = htonl(peer->params.id);
@@ -543,7 +610,7 @@ static int netjack2_send_opus(struct netjack2_peer *peer, uint32_t nframes,
 		}
 	}
 
-	strcpy(header.type, "header");
+	strncpy(header.type, "header", sizeof(header.type));
 	header.data_type = htonl('a');
 	header.data_stream = htonl(peer->our_stream);
 	header.id = htonl(peer->params.id);
@@ -611,7 +678,7 @@ static int netjack2_send_int(struct netjack2_peer *peer, uint32_t nframes,
 			memset(ap, 0, max_encoded);
 	}
 
-	strcpy(header.type, "header");
+	strncpy(header.type, "header", sizeof(header.type));
 	header.data_type = htonl('a');
 	header.data_stream = htonl(peer->our_stream);
 	header.id = htonl(peer->params.id);
@@ -691,7 +758,7 @@ static inline int32_t netjack2_driver_sync_wait(struct netjack2_peer *peer)
 
 receive_error:
 	pw_log_warn("recv error: %m");
-	return 0;
+	return -errno;
 }
 
 static inline int32_t netjack2_manager_sync_wait(struct netjack2_peer *peer)
@@ -735,7 +802,7 @@ static inline int32_t netjack2_manager_sync_wait(struct netjack2_peer *peer)
 
 receive_error:
 	pw_log_warn("recv error: %m");
-	return 0;
+	return -errno;
 }
 
 static int netjack2_recv_midi(struct netjack2_peer *peer, struct nj2_packet_header *header, uint32_t *count,
@@ -1031,7 +1098,7 @@ static int netjack2_recv_data(struct netjack2_peer *peer,
 	}
 	for (i = 0; i < n_midi; i++) {
 		if (!midi[i].filled && midi[i].data != NULL)
-			netjack2_to_midi(midi[i].data, peer->params.period_size * sizeof(float), NULL);
+			netjack2_clear_midi(midi[i].data, peer->params.period_size * sizeof(float));
 	}
 	peer->sync.cycle = ntohl(header.cycle);
 	return 0;
