@@ -11,6 +11,45 @@
 #include "../defs.h"
 #include "../module.h"
 
+/** \page page_pulse_module_echo_cancel Echo Cancel
+ *
+ * ## Module Name
+ *
+ * `module-echo-cancel`
+ *
+ * ## Module Options
+ *
+ * @pulse_module_options@
+ *
+ * ## See Also
+ *
+ * \ref page_module_echo_cancel "libpipewire-module-echo-cancel"
+ */
+
+static const char *const pulse_module_options =
+	"source_name=<name for the source> "
+	"source_properties=<properties for the source> "
+	"source_master=<name of source to filter> "
+	"sink_name=<name for the sink> "
+	"sink_properties=<properties for the sink> "
+	"sink_master=<name of sink to filter> "
+	"rate=<sample rate> "
+	"channels=<number of channels> "
+	"channel_map=<channel map> "
+	"aec_method=<implementation to use> "
+	"aec_args=<parameters for the AEC engine> ";
+#if 0
+	/* These are not implemented because they don't
+	 * really make sense in the PipeWire context */
+	"format=<sample format> "
+	"adjust_time=<how often to readjust rates in s> "
+	"adjust_threshold=<how much drift to readjust after in ms> "
+	"autoloaded=<set if this module is being loaded automatically> "
+	"save_aec=<save AEC data in /tmp> "
+	"use_volume_sharing=<yes or no> "
+	"use_master_format=<yes or no> "
+#endif
+
 #define NAME "echo-cancel"
 
 PW_LOG_TOPIC_STATIC(mod_topic, "mod." NAME);
@@ -114,29 +153,7 @@ static int module_echo_cancel_unload(struct module *module)
 static const struct spa_dict_item module_echo_cancel_info[] = {
 	{ PW_KEY_MODULE_AUTHOR, "Arun Raghavan <arun@asymptotic.io>" },
 	{ PW_KEY_MODULE_DESCRIPTION, "Acoustic echo canceller" },
-	{ PW_KEY_MODULE_USAGE, "source_name=<name for the source> "
-				"source_properties=<properties for the source> "
-				"source_master=<name of source to filter> "
-				"sink_name=<name for the sink> "
-				"sink_properties=<properties for the sink> "
-				"sink_master=<name of sink to filter> "
-				"rate=<sample rate> "
-				"channels=<number of channels> "
-				"channel_map=<channel map> "
-				"aec_method=<implementation to use> "
-				"aec_args=<parameters for the AEC engine> "
-#if 0
-				/* These are not implemented because they don't
-				 * really make sense in the PipeWire context */
-				"format=<sample format> "
-				"adjust_time=<how often to readjust rates in s> "
-				"adjust_threshold=<how much drift to readjust after in ms> "
-				"autoloaded=<set if this module is being loaded automatically> "
-				"save_aec=<save AEC data in /tmp> "
-				"use_volume_sharing=<yes or no> "
-				"use_master_format=<yes or no> "
-#endif
-	},
+	{ PW_KEY_MODULE_USAGE, pulse_module_options },
 	{ PW_KEY_MODULE_VERSION, PACKAGE_VERSION },
 };
 
@@ -159,7 +176,7 @@ static int parse_point(const char **point, float f[3])
 static int rename_geometry(struct pw_properties *props, const char *pa_key, const char *pw_key)
 {
 	const char *str;
-	int len;
+	int i = 0, len;
 	char *args;
 	size_t size;
 	FILE *f;
@@ -172,19 +189,26 @@ static int rename_geometry(struct pw_properties *props, const char *pa_key, cons
 	if ((f = open_memstream(&args, &size)) == NULL)
 		return -errno;
 
-	fprintf(f, "[ ");
+	fprintf(f, "[");
 	while (true) {
 		float p[3];
+		char ps0[64], ps1[64], ps2[64];
 		if ((len = parse_point(&str, p)) < 0)
 			break;
 
-		fprintf(f, "[ %f %f %f ] ", p[0], p[1], p[2]);
+		pw_log_info("Got mic #%d position: (%g, %g, %g)", i, p[0], p[1], p[2]);
+
+		fprintf(f, "%s [ %s, %s, %s ]", i == 0 ?  "" : ",",
+				spa_dtoa(ps0, sizeof(ps0), p[0]),
+				spa_dtoa(ps1, sizeof(ps1), p[1]),
+				spa_dtoa(ps2, sizeof(ps2), p[2]));
 		str += len;
 		if (*str != ',')
 			break;
 		str++;
+		i++;
 	}
-	fprintf(f, "]");
+	fprintf(f, " ]");
 	fclose(f);
 
 	pw_properties_set(props, pw_key, args);
@@ -199,6 +223,7 @@ static int rename_direction(struct pw_properties *props, const char *pa_key, con
 	const char *str;
 	int res;
 	float f[3];
+	char fs0[64], fs1[64], fs2[64];
 
 	if ((str = pw_properties_get(props, pa_key)) == NULL)
 		return 0;
@@ -208,7 +233,12 @@ static int rename_direction(struct pw_properties *props, const char *pa_key, con
 	if ((res = parse_point(&str, f)) < 0)
 		return res;
 
-	pw_properties_setf(props, pw_key, "[ %f %f %f ]", f[0], f[1], f[2]);
+	pw_log_info("Got target direction: (%g, %g, %g)", f[0], f[1], f[2]);
+
+	pw_properties_setf(props, pw_key, "[ %s, %s, %s ]",
+				spa_dtoa(fs0, sizeof(fs0), f[0]),
+				spa_dtoa(fs1, sizeof(fs1), f[1]),
+				spa_dtoa(fs2, sizeof(fs2), f[2]));
 	pw_properties_set(props, pa_key, NULL);
 	return 0;
 }
@@ -302,8 +332,14 @@ static int module_echo_cancel_prepare(struct module * const module)
 			rename_bool_prop(aec_props, "extended_filter", "webrtc.extended_filter");
 			rename_bool_prop(aec_props, "experimental_agc", "webrtc.experimental_agc");
 			rename_bool_prop(aec_props, "beamforming", "webrtc.beamforming");
-			rename_geometry(aec_props, "mic_geometry", "webrtc.mic-geometry");
-			rename_direction(aec_props, "target_direction", "webrtc.target-direction");
+			if ((res = rename_geometry(aec_props, "mic_geometry", "webrtc.mic-geometry")) < 0) {
+				pw_log_warn("failed to parse mic_geometry: %s", spa_strerror(res));
+				goto out;
+			}
+			if ((res = rename_direction(aec_props, "target_direction", "webrtc.target-direction")) < 0) {
+				pw_log_warn("failed to parse target_direction: %s", spa_strerror(res));
+				goto out;
+			}
 		}
 		pw_properties_set(props, "aec_args", NULL);
 	}
