@@ -565,19 +565,18 @@ static int parse_spa_libs(void *user_data, const char *location,
 {
 	struct data *d = user_data;
 	struct pw_context *context = d->context;
-	struct spa_json it[2];
+	struct spa_json it[1];
 	char key[512], value[512];
 	int res;
 
-	spa_json_init(&it[0], str, len);
-	if (spa_json_enter_object(&it[0], &it[1]) < 0) {
+	if (spa_json_begin_object(&it[0], str, len) < 0) {
 		pw_log_error("config file error: context.spa-libs is not an "
 				"object in '%.*s'", (int)len, str);
 		return -EINVAL;
 	}
 
-	while (spa_json_get_string(&it[1], key, sizeof(key)) > 0) {
-		if (spa_json_get_string(&it[1], value, sizeof(value)) > 0) {
+	while (spa_json_get_string(&it[0], key, sizeof(key)) > 0) {
+		if (spa_json_get_string(&it[0], value, sizeof(value)) > 0) {
 			if ((res = pw_context_add_spa_lib(context, key, value)) < 0) {
 				pw_log_error("error adding spa-libs for '%s' in '%.*s': %s",
 					key, (int)len, str, spa_strerror(res));
@@ -629,7 +628,8 @@ static int load_module(struct pw_context *context, const char *key, const char *
  *  "!null" -> same as !null
  *  !"null" and "!\"null\"" matches anything that is not the string "null"
  */
-static bool find_match(struct spa_json *arr, const struct spa_dict *props, bool condition)
+SPA_EXPORT
+bool pw_conf_find_match(struct spa_json *arr, const struct spa_dict *props, bool condition)
 {
 	struct spa_json it[1];
 	const char *as = arr->cur;
@@ -641,16 +641,10 @@ static bool find_match(struct spa_json *arr, const struct spa_dict *props, bool
 		int match = 0, fail = 0;
 		int len;
 
-		while (spa_json_get_string(&it[0], key, sizeof(key)) > 0) {
+		while ((len = spa_json_object_next(&it[0], key, sizeof(key), &value)) > 0) {
 			bool success = false, is_null, reg = false, parse_string = true;
 			int skip = 0;
 
-			if ((len = spa_json_next(&it[0], &value)) <= 0) {
-				pw_log_warn("malformed match rule: key '%s' has "
-						"no value in '%.*s'", key, az, as);
-				break;
-			}
-
 			/* first decode a string, when there was a string, we assume it
 			 * can not be null but the "null" string, unless there is a modifier,
 			 * see below. */
@@ -752,44 +746,36 @@ static int parse_modules(void *user_data, const char *location,
 {
 	struct data *d = user_data;
 	struct pw_context *context = d->context;
-	struct spa_json it[4];
+	struct spa_json it[3];
 	char key[512];
 	int res = 0, r;
 
 	spa_autofree char *s = strndup(str, len);
-	spa_json_init(&it[0], s, len);
-	if (spa_json_enter_array(&it[0], &it[1]) < 0) {
+	if (spa_json_begin_array(&it[0], s, len) < 0) {
 		pw_log_error("context.modules is not an array in '%.*s'",
 				(int)len, str);
 		return -EINVAL;
 	}
 
-	while ((r = spa_json_enter_object(&it[1], &it[2])) > 0) {
+	while ((r = spa_json_enter_object(&it[0], &it[1])) > 0) {
 		char *name = NULL, *args = NULL, *flags = NULL;
 		bool have_match = true;
+		const char *val;
+		int l;
 
-		while (spa_json_get_string(&it[2], key, sizeof(key)) > 0) {
-			const char *val;
-			int l;
-
-			if ((l = spa_json_next(&it[2], &val)) <= 0) {
-				pw_log_warn("malformed module: key '%s' has no "
-						"value in '%.*s'", key, (int)len, str);
-				break;
-			}
-
+		while ((l = spa_json_object_next(&it[1], key, sizeof(key), &val)) > 0) {
 			if (spa_streq(key, "name")) {
 				name = (char*)val;
 				spa_json_parse_stringn(val, l, name, l+1);
 			} else if (spa_streq(key, "args")) {
 				if (spa_json_is_container(val, l))
-					l = spa_json_container_len(&it[2], val, l);
+					l = spa_json_container_len(&it[1], val, l);
 
 				args = (char*)val;
 				spa_json_parse_stringn(val, l, args, l+1);
 			} else if (spa_streq(key, "flags")) {
 				if (spa_json_is_container(val, l))
-					l = spa_json_container_len(&it[2], val, l);
+					l = spa_json_container_len(&it[1], val, l);
 				flags = (char*)val;
 				spa_json_parse_stringn(val, l, flags, l+1);
 			} else if (spa_streq(key, "condition")) {
@@ -798,8 +784,8 @@ static int parse_modules(void *user_data, const char *location,
 							(int)len, str);
 					break;
 				}
-				spa_json_enter(&it[2], &it[3]);
-				have_match = find_match(&it[3], &context->properties->dict, true);
+				spa_json_enter(&it[1], &it[2]);
+				have_match = pw_conf_find_match(&it[2], &context->properties->dict, true);
 			} else {
 				pw_log_warn("unknown module key '%s' in '%.*s'", key,
 						(int)len, str);
@@ -862,43 +848,35 @@ static int parse_objects(void *user_data, const char *location,
 {
 	struct data *d = user_data;
 	struct pw_context *context = d->context;
-	struct spa_json it[4];
+	struct spa_json it[3];
 	char key[512];
 	int res = 0, r;
 
 	spa_autofree char *s = strndup(str, len);
-	spa_json_init(&it[0], s, len);
-	if (spa_json_enter_array(&it[0], &it[1]) < 0) {
+	if (spa_json_begin_array(&it[0], s, len) < 0) {
 		pw_log_error("config file error: context.objects is not an array");
 		return -EINVAL;
 	}
 
-	while ((r = spa_json_enter_object(&it[1], &it[2])) > 0) {
+	while ((r = spa_json_enter_object(&it[0], &it[1])) > 0) {
 		char *factory = NULL, *args = NULL, *flags = NULL;
 		bool have_match = true;
+		const char *val;
+		int l;
 
-		while (spa_json_get_string(&it[2], key, sizeof(key)) > 0) {
-			const char *val;
-			int l;
-
-			if ((l = spa_json_next(&it[2], &val)) <= 0) {
-				pw_log_warn("malformed object: key '%s' has no "
-						"value in '%.*s'", key, (int)len, str);
-				break;
-			}
-
+		while ((l = spa_json_object_next(&it[1], key, sizeof(key), &val)) > 0) {
 			if (spa_streq(key, "factory")) {
 				factory = (char*)val;
 				spa_json_parse_stringn(val, l, factory, l+1);
 			} else if (spa_streq(key, "args")) {
 				if (spa_json_is_container(val, l))
-					l = spa_json_container_len(&it[2], val, l);
+					l = spa_json_container_len(&it[1], val, l);
 
 				args = (char*)val;
 				spa_json_parse_stringn(val, l, args, l+1);
 			} else if (spa_streq(key, "flags")) {
 				if (spa_json_is_container(val, l))
-					l = spa_json_container_len(&it[2], val, l);
+					l = spa_json_container_len(&it[1], val, l);
 
 				flags = (char*)val;
 				spa_json_parse_stringn(val, l, flags, l+1);
@@ -908,8 +886,8 @@ static int parse_objects(void *user_data, const char *location,
 							(int)len, str);
 					break;
 				}
-				spa_json_enter(&it[2], &it[3]);
-				have_match = find_match(&it[3], &context->properties->dict, true);
+				spa_json_enter(&it[1], &it[2]);
+				have_match = pw_conf_find_match(&it[2], &context->properties->dict, true);
 			} else {
 				pw_log_warn("unknown object key '%s' in '%.*s'", key,
 						(int)len, str);
@@ -945,9 +923,10 @@ static char **pw_strv_insert_at(char **strv, int len, int pos, const char *str)
 
 	n = realloc(strv, sizeof(char*) * (len + 2));
 	if (n == NULL) {
-		free(strv);
+		pw_free_strv(strv);
 		return NULL;
 	}
+
 	strv = n;
 
 	memmove(strv+pos+1, strv+pos, sizeof(char*) * (len+1-pos));
@@ -955,15 +934,16 @@ static char **pw_strv_insert_at(char **strv, int len, int pos, const char *str)
 	return strv;
 }
 
-static int do_exec(struct pw_context *context, const char *path, const char *args)
+static int do_exec(struct pw_context *context, char *const *argv)
 {
 	int pid, res;
 
 	pid = fork();
 
 	if (pid == 0) {
-		char **arg;
-		int n_args;
+		char buf[1024];
+		char *const *p;
+		struct spa_strbuf s;
 
 		/* Double fork to avoid zombies; we don't want to set SIGCHLD handler */
 		pid = fork();
@@ -975,23 +955,16 @@ static int do_exec(struct pw_context *context, const char *path, const char *arg
 			exit(0);
 		}
 
-		arg = pw_strv_parse(args, strlen(args), INT_MAX, &n_args);
-		if (arg == NULL) {
-			pw_log_error("error parsing arguments: %m");
-			goto done;
-		}
-		arg = pw_strv_insert_at(arg, n_args, 0, path);
-		if (arg == NULL) {
-			pw_log_error("error constructing arguments: %m");
-			goto done;
-		}
-		pw_log_info("exec %s '%s'", path, args);
-		res = execvp(path, arg);
-		pw_free_strv(arg);
+		spa_strbuf_init(&s, buf, sizeof(buf));
+		for (p = argv; *p; ++p)
+			spa_strbuf_append(&s, " '%s'", *p);
+
+		pw_log_info("exec%s", s.buffer);
+		res = execvp(argv[0], argv);
 
 		if (res == -1) {
 			res = -errno;
-			pw_log_error("execvp error '%s': %m", path);
+			pw_log_error("execvp error '%s': %m", argv[0]);
 		}
 done:
 		exit(1);
@@ -1016,53 +989,75 @@ done:
  *   }
  * ]
  */
+
+static char **make_exec_argv(const char *path, const char *value, int len)
+{
+	char **argv = NULL;
+	int n_args;
+
+	if (spa_json_is_container(value, len)) {
+		if (!spa_json_is_array(value, len)) {
+			errno = EINVAL;
+			return NULL;
+		}
+		argv = pw_strv_parse(value, len, INT_MAX, &n_args);
+	} else {
+		spa_autofree char *s = calloc(1, len + 1);
+		if (!s)
+			return NULL;
+
+		if (spa_json_parse_stringn(value, len, s, len + 1) < 0) {
+			errno = EINVAL;
+			return NULL;
+		}
+		argv = pw_split_strv(s, " \t", INT_MAX, &n_args);
+	}
+
+	if (!argv)
+		return NULL;
+
+	return pw_strv_insert_at(argv, n_args, 0, path);
+}
+
 static int parse_exec(void *user_data, const char *location,
 		const char *section, const char *str, size_t len)
 {
 	struct data *d = user_data;
 	struct pw_context *context = d->context;
-	struct spa_json it[4];
+	struct spa_json it[3];
 	char key[512];
 	int r, res = 0;
 
 	spa_autofree char *s = strndup(str, len);
-	spa_json_init(&it[0], s, len);
-	if (spa_json_enter_array(&it[0], &it[1]) < 0) {
+	if (spa_json_begin_array(&it[0], s, len) < 0) {
 		pw_log_error("config file error: context.exec is not an array in '%.*s'",
 				(int)len, str);
 		return -EINVAL;
 	}
 
-	while ((r = spa_json_enter_object(&it[1], &it[2])) > 0) {
-		char *path = NULL, *args = NULL;
+	while ((r = spa_json_enter_object(&it[0], &it[1])) > 0) {
+		char *path = NULL;
+		const char *args_val = "[]", *val;
+		int args_len = 2, l;
 		bool have_match = true;
 
-		while (spa_json_get_string(&it[2], key, sizeof(key)) > 0) {
-			const char *val;
-			int l;
-
-			if ((l = spa_json_next(&it[2], &val)) <= 0) {
-				pw_log_warn("malformed exec: key '%s' has no "
-						"value in '%.*s'", key, (int)len, str);
-				break;
-			}
-
+		while ((l = spa_json_object_next(&it[1], key, sizeof(key), &val)) > 0) {
 			if (spa_streq(key, "path")) {
 				path = (char*)val;
 				spa_json_parse_stringn(val, l, path, l+1);
 			} else if (spa_streq(key, "args")) {
 				if (spa_json_is_container(val, l))
-					l = spa_json_container_len(&it[2], val, l);
-				args = (char*)val;
-				spa_json_parse_stringn(val, l, args, l+1);
+					l = spa_json_container_len(&it[1], val, l);
+				args_val = val;
+				args_len = l;
 			} else if (spa_streq(key, "condition")) {
 				if (!spa_json_is_array(val, l)) {
 					pw_log_warn("expected array for condition in '%.*s'",
 							(int)len, str);
-					break;
+					goto next;
 				}
-				spa_json_enter(&it[2], &it[3]);
-				have_match = find_match(&it[3], &context->properties->dict, true);
+				spa_json_enter(&it[1], &it[2]);
+				have_match = pw_conf_find_match(&it[2], &context->properties->dict, true);
 			} else {
 				pw_log_warn("unknown exec key '%s' in '%.*s'", key,
 						(int)len, str);
@@ -1072,11 +1067,23 @@ static int parse_exec(void *user_data, const char *location,
 			continue;
 
 		if (path != NULL) {
-			res = do_exec(context, path, args);
+			char **argv = make_exec_argv(path, args_val, args_len);
+
+			if (!argv) {
+				pw_log_warn("expected array or string for args in '%.*s'",
+						(int)len, str);
+				goto next;
+			}
+
+			res = do_exec(context, argv);
+			pw_free_strv(argv);
 			if (res < 0)
 				break;
 			d->count++;
 		}
+
+	next:
+		continue;
 	}
 	if (r < 0)
 		pw_log_warn("malformed object array in '%.*s'", (int)len, str);
@@ -1193,6 +1200,10 @@ int pw_conf_load_conf_for_context(struct pw_properties *props, struct pw_propert
 	conf_name = getenv("PIPEWIRE_CONFIG_NAME");
 	if ((res = try_load_conf(conf_prefix, conf_name, conf)) < 0) {
 		conf_name = pw_properties_get(props, PW_KEY_CONFIG_NAME);
+		if (spa_streq(conf_name, "client-rt.conf")) {
+			pw_log_warn("setting config.name to client-rt.conf is deprecated, using client.conf");
+			conf_name = NULL;
+		}
 		if (conf_name == NULL)
 			conf_name = "client.conf";
 		else if (!valid_conf_name(conf_name)) {
@@ -1268,43 +1279,41 @@ int pw_conf_match_rules(const char *str, size_t len, const char *location,
 		void *data)
 {
 	const char *val;
-	struct spa_json it[4], actions;
+	struct spa_json it[3], actions;
 	int r;
 
-	spa_json_init(&it[0], str, len);
-	if (spa_json_enter_array(&it[0], &it[1]) < 0) {
+	if (spa_json_begin_array(&it[0], str, len) < 0) {
 		pw_log_warn("expect array of match rules in: '%.*s'", (int)len, str);
 		return 0;
 	}
 
-	while ((r = spa_json_enter_object(&it[1], &it[2])) > 0) {
+	while ((r = spa_json_enter_object(&it[0], &it[1])) > 0) {
 		char key[64];
 		bool have_match = false, have_actions = false;
+		int res, l;
 
-		while (spa_json_get_string(&it[2], key, sizeof(key)) > 0) {
+		while ((l = spa_json_object_next(&it[1], key, sizeof(key), &val)) > 0) {
 			if (spa_streq(key, "matches")) {
-				if (spa_json_enter_array(&it[2], &it[3]) < 0) {
+				if (!spa_json_is_array(val, l)) {
 					pw_log_warn("expected array as matches in '%.*s'",
 							(int)len, str);
 					break;
 				}
 
-				have_match = find_match(&it[3], props, false);
+				spa_json_enter(&it[1], &it[2]);
+				have_match = pw_conf_find_match(&it[2], props, false);
 			}
 			else if (spa_streq(key, "actions")) {
-				if (spa_json_enter_object(&it[2], &actions) > 0)
-					have_actions = true;
-				else
+				if (!spa_json_is_object(val, l)) {
 					pw_log_warn("expected object as match actions in '%.*s'",
 							(int)len, str);
+				} else {
+					have_actions = true;
+					spa_json_enter(&it[1], &actions);
+				}
 			}
 			else {
 				pw_log_warn("unknown match key '%s'", key);
-				if (spa_json_next(&it[2], &val) <= 0) {
-					pw_log_warn("malformed match rule: key '%s' has "
-							"no value in '%.*s'", key, (int)len, str);
-					break;
-				}
 			}
 		}
 		if (!have_match)
@@ -1314,16 +1323,9 @@ int pw_conf_match_rules(const char *str, size_t len, const char *location,
 			continue;
 		}
 
-		while (spa_json_get_string(&actions, key, sizeof(key)) > 0) {
-			int res, len;
+		while ((len = spa_json_object_next(&actions, key, sizeof(key), &val)) > 0) {
 			pw_log_debug("action %s", key);
 
-			if ((len = spa_json_next(&actions, &val)) <= 0) {
-				pw_log_warn("malformed action: key '%s' has no value in '%.*s'",
-						key, (int)len, str);
-				break;
-			}
-
 			if (spa_json_is_container(val, len))
 				len = spa_json_container_len(&actions, val, len);
 
